@@ -5,20 +5,27 @@
 #' @param y Vector of observations
 #' @param X_mu Model matrix for the expected values \eqn{\mu}
 #' @param X_sigma Model matrix for the logarithms of the standard deviations \eqn{\sigma}
-#' @param check_hessian Boolean, if TRUE it is checked whether the Hessian is negative-definite
 #' @param slvr_options A list with options to be passed on to the function
-#' \code{maxNR} from the package \code{maxLik} which maximizes the log-liklihood
-#' @param slvr_log Boolean, if TRUE the output of \code{maxNR} is added to the 'lmvar' object
+#' \code{\link[maxLik]{maxLik}} which maximizes the log-liklihood
 #' @param intercept_mu Boolean, if TRUE a column with the name '(Intercept)' is added to the matrix \code{X_mu}. This column
 #' represents the intercept term in the model for \eqn{\mu}.
 #' @param intercept_sigma Boolean, if TRUE a column with the name '(Intercept_s)' is added to the matrix \code{X_sigma}. This column
 #' represents the intercept term in the model for \eqn{\log \sigma}.
+#' @param control A list with further options. The following options can be set.
+#' \itemize{
+#' \item \code{check_hessian} Boolean, if TRUE it is checked whether the Hessian is negative-definite, i.e., whether the
+#' log-likelihood is at a maximum. The default value is TRUE.
+#' \item \code{slvr_log} Boolean, if TRUE the output of \code{maxLik} is added to the 'lmvar' object.
+#' The default value is FALSE.
+#' \item \code{running_diagnostics} Boolean, if TRUE diagnostic messages about errors that occur will be
+#' printed during the run. The default value is FALSE.
+#' }
 #' @param ... Additional arguments, not used in the current implementation
 #'
 #' @return An object of class 'lmvar'.
 #'
 #' @details If the matrix \code{X_mu} is not specified, the model for the expected values \eqn{\mu} will consist of an intercept term only.
-#' Likewise, the model for \eqn{\log \sigma} will consist of an intercept term only if \code{X_sigma} is not specified.
+#' The argument \code{intercept_mu} is ignored in this case. Likewise, the model for \eqn{\log \sigma} will consist of an intercept term only if \code{X_sigma} is not specified.
 #' In the latter case, the model reduces to a standard linear model.
 #'
 #' The input matrices can be of class \code{\link{matrix}} or of class \link[Matrix:Matrix-class]{Matrix}. They can also be of
@@ -46,25 +53,38 @@
 #' Matrix \code{X_mu} can not have a column with the name '(Intercept)'.
 #' Matrix \code{X_sigma} can not have a column with the name '(Intercept_s)'. Both names are reserved names.
 #'
-#' The function \code{maxNR} from the \code{maxLik} package, is used to maximize the (profile) log-likelihood.
-#' \code{maxNR} returns a
-#' termination code which reports whether a maximum was found, see \code{\link[maxLik]{maxNR}}.
-#' A potential problem is reported by a termination code different from \code{1}, \code{2} or \code{8}. In case of a different
-#' termination code, the message from \code{maxNR} is output as a warning.
+#' The function \code{\link[maxLik]{maxLik}} from the \code{maxLik} package, is used to maximize the (profile) log-likelihood.
+#' \code{maxLik} returns a
+#' termination code which reports whether a maximum was found, see \code{maxLik}.
+#' For the method "NR", a potential problem is reported by a \code{maxLik} return code different from \code{1}, \code{2} or \code{8}.
+#' For other methods, a code different from \code{0} flags a potential problem.
+#' In case the return code flags a potential problem, the message from \code{maxLik} is output as a warning.
 #'
-#' All list elements in \code{slvr_options} are passed on as arguments to \code{maxNR}. The name of the list element is
+#' All list elements in \code{slvr_options} are passed on as arguments to \code{maxLik}. The name of the list element is
 #' the argument name, the value of the list element is the argument value.
-#' It is not allowed to pass on the arguments \code{fn}, \code{grad}
-#' or \code{hess}.
+#' It is not allowed to pass on the arguments \code{fn}, \code{grad} or \code{hess}. In case the list does not contain an element
+#' \code{method}, it is set to "NR". In case the list does not contain an element \code{start}, an initial estimate for
+#' \eqn{\beta_\sigma} is set by \code{lmvar}.
 #'
 #' In case one wants to supply an initial estimate for the coefficients, one has to supply an initial estimate
-#' for \eqn{\beta_\sigma}. If \code{beta_sigma_initial} is the initial estimate, one pass on the argument
+#' for \eqn{\beta_\sigma}. If \code{beta_sigma_initial} is the initial estimate, one passes on the argument
 #' \code{slvr_options = list(start = beta_sigma_initial)}.
-#' The inital estimate \code{beta_sigma_initial} must be a numeric vector with a length equal to the number
-#' of columns of \code{X_sigma} plus one. The first element of the vector is the initial estimate of the intercept term for
-#' \eqn{sigma}, the second element is the inital estimate corresponding to the first column in \code{X_sigma}, etc.
+#' The inital estimate \code{beta_sigma_initial} must be a numeric vector. Its length must be as follows.
+#' \itemize{
+#' \item In case \code{X_sigma} is not specified or has the value \code{NULL}, the initial estimate must be
+#' a single value.
+#' \item In case \code{X_sigma} is specified and \code{intercept_sigma = TRUE}: the length must be equal to the number
+#' of columns of \code{X_sigma} plus one. The first element of the vector is the initial estimate of the
+#' intercept term for \eqn{\log \sigma}, the second element is the inital estimate corresponding to the first
+#' column in \code{X_sigma}, the third element is the inital estimate corresponding to the second
+#' column in \code{X_sigma}, etc.
+#' \item  In case \code{X_sigma} is specified and \code{intercept_sigma = FALSE}: the length must be equal to the
+#' number of columns of \code{X_sigma}. The first element of the vector is the initial estimate corresponding to the
+#' first column in \code{X_sigma}, the second element is the inital estimate corresponding to the second
+#' column in \code{X_sigma}, etc.
+#' }
 #'
-#' It is not possible (and there is no need) to supply an inital estimate for \eqn{\beta_\mu}.
+#' There is no need to supply an inital estimate for \eqn{\beta_\mu}, see the vignette 'Math' for details.
 #'
 #' A \code{lmvar} object is a list. Users are discouraged to access list-members directly.
 #' Instead, list-members are to be accessed with the various accessor and utility functions in the package.
@@ -74,11 +94,16 @@
 #' \item \code{object$X_mu} the  model matrix for \eqn{\mu}. In general, it differs from the user-supplied \code{X_mu} because
 #' \code{lmvar} adds an intercept-column and makes the matrix full-rank.
 #' \item \code{object$X_sigma} the  model matrix for \eqn{\log \sigma}. In general, it differs from the user-supplied
-#' \code{X_sigma} because
-#' \code{lmvar} adds an intercept-column and makes the matrix full-rank.
-#' \item \code{slvr_log} the output of \code{maxNR} (the solver routine used to maximize the likelihood). Included only
+#' \code{X_sigma} because \code{lmvar} adds an intercept-column and makes the matrix full-rank.
+#' \item \code{intercept_mu} boolean which tells whether or not an intercept column \code{(Intercept)} has been added to the
+#' model matrix \eqn{X_\mu}
+#' \item \code{intercept_sigma} boolean which tells whether or not an intercept column \code{(Intercept_s)} has been added to the
+#' model matrix \eqn{X_\sigma}
+#' \item \code{slvr_options} the value of the argument \code{slvr_options} in the call of \code{lmvar}
+#' \item \code{control} the value of the argument \code{control} in the call of \code{lmvar}
+#' \item \code{slvr_log} the output of \code{maxLik} (the solver routine used to maximize the likelihood). Included only
 #' if the argument \code{slvr_log} has the value \code{TRUE}.
-#' See \code{\link[maxLik]{maxNR}} for details about this output.
+#' See \code{\link[maxLik]{maxLik}} for details about this output.
 #' }
 #'
 #' See the vignettes
@@ -88,40 +113,52 @@
 #'
 #' @example R/examples/lmvar_examples.R
 #'
-lmvar <- function( y, X_mu = NULL, X_sigma = NULL, check_hessian = TRUE, slvr_options = list(), slvr_log = FALSE,
-                   intercept_mu = TRUE, intercept_sigma = TRUE, ...){
+lmvar <- function( y, X_mu = NULL, X_sigma = NULL,
+                   intercept_mu = TRUE, intercept_sigma = TRUE, slvr_options = list(method = "NR"), control = list(), ...){
 
-  logLHood <- function(beta_sigma){
+  print_diagnostics <- function( beta_sigma, sigma_inv){
+
+    i = which.min(beta_sigma)
+    name = colnames(X_sigma)[i]
+    value = beta_sigma[i]
+    message( "    beta_sigma smallest: '", name, "' = ", signif( value, 3))
+
+    i = which.max(beta_sigma)
+    name = colnames(X_sigma)[i]
+    value = beta_sigma[i]
+    message( "                largest: '", name, "' = ", signif( value, 3))
+    cat("\n")
+
+    i = which.min(sigma_inv)
+    value = sigma_inv[i]
+    message( "    sigma inverse smallest: ", signif( value, 3), " for observation ", i)
+
+    i = which.max(sigma_inv)
+    value = sigma_inv[i]
+    message( "                   largest: ", signif( value, 3), " for observation ", i)
+    cat("\n")
+  }
+
+  logLHood <- function(beta_sigma, return_hessian = FALSE){
 
     sigma_inv = as.numeric(exp(- X_sigma %*% beta_sigma))
     M = X_mu * sigma_inv
     M = tryCatch( chol2inv(chol(Matrix::t(M) %*% M)),          # Exploit that t(M) %*% M is symmetric and positive-definite
 
                   error = function(e){
-                    i = which.min(beta_sigma)
-                    name = colnames(X_sigma)[i]
-                    value = beta_sigma[i]
-                    message( "Smallest element of beta_sigma is '", name, "' with value ", signif( value, 3))
 
-                    i = which.max(beta_sigma)
-                    name = colnames(X_sigma)[i]
-                    value = beta_sigma[i]
-                    message( "Largest element of beta_sigma is '", name, "' with value ", signif( value, 3))
-
-                    i = which.min(sigma_inv)
-                    value = sigma_inv[i]
-                    message( "Smallest element of sigma inverse is ", signif( value, 3), " for observation ", i)
-
-                    i = which.max(sigma_inv)
-                    value = sigma_inv[i]
-                    message( "Largest element of sigma inverse is ", signif( value, 3), " for observation ", i)
-                    terms = X_sigma[i,] * beta_sigma
-                    i = which.max(terms)
-                    message("   largest contribution to predictor is ", signif( terms[i], 3), " for covariate ", colnames(X_sigma)[i],
-                            " with beta is ", signif( beta_sigma[i], 3))
-                    stop(e)
+                    if (control$running_diagnostics){
+                      message (e)
+                      cat("\n")
+                      print_diagnostics( beta_sigma, sigma_inv)
+                    }
+                    return(NA)
                   })
 
+    if(class(M) == 'logical'){
+      return (NA)
+    }
+
     beta_mu = M %*% Matrix::t(X_mu) %*% (y * sigma_inv * sigma_inv)
     mu = as.numeric(X_mu %*% beta_mu)
     res = (y - mu) * sigma_inv
@@ -130,17 +167,40 @@ lmvar <- function( y, X_mu = NULL, X_sigma = NULL, check_hessian = TRUE, slvr_op
     logLik= -0.5*n*log(2*pi) + sum(log(sigma_inv)) - 0.5*sum(res*res)
 
     # Calculate gradient
-    dBeta_sigma = as.numeric(Matrix::t(X_sigma) %*% (res*res - 1))  # gradient of log-likelihood w.r.t. beta_sigma
+    if(!(slvr_options$method %in% c( "NM", "SANN"))){
+
+      dBeta_sigma = as.numeric(Matrix::t(X_sigma) %*% (res*res - 1))  # gradient of log-likelihood w.r.t. beta_sigma
+
+      # Set attribute
+      attr( logLik, "gradient") = dBeta_sigma
+    }
 
     # Calculate Hessian
-    XT = chol(M) %*% Matrix::t(X_mu) %*% (X_sigma * (res * sigma_inv)) # Use Choleski decomposition to have H_ss_1 symmetric
-    H_ss_1 = 4 * Matrix::t(XT) %*% XT
-    XT = X_sigma * abs(res)
-    H_ss_2 = -2 * Matrix::t(XT) %*% XT
+    if(slvr_options$method == "NR" | return_hessian){
 
-    # Set attributes
-    attr( logLik, "gradient") = dBeta_sigma
-    attr( logLik, "hessian") = H_ss_1 + H_ss_2
+      M = tryCatch( chol(M),
+
+                    error = function(e){
+
+                      if (control$running_diagnostics){
+                        message(e)
+                        cat("\n")
+                        print_diagnostics( beta_sigma, sigma_inv)
+                      }
+                      return(NA)
+                   })
+      if(class(M) == 'logical'){
+        return (NA)
+      }
+
+      XT = M %*% Matrix::t(X_mu) %*% (X_sigma * (res * sigma_inv)) # Use Choleski decomposition to have H_ss_1 symmetric
+      H_ss_1 = 4 * Matrix::t(XT) %*% XT
+      XT = X_sigma * abs(res)
+      H_ss_2 = -2 * Matrix::t(XT) %*% XT
+
+      # Set attribute
+      attr( logLik, "hessian") = as.matrix(H_ss_1 + H_ss_2)
+    }
 
     return(logLik)
   }
@@ -165,6 +225,25 @@ lmvar <- function( y, X_mu = NULL, X_sigma = NULL, check_hessian = TRUE, slvr_op
     X_sigma = as.matrix(X_sigma)
   }
 
+  # set default solve method
+  if (!("method" %in% names(slvr_options))){
+    slvr_options$method = "NR"
+  }
+
+  # Make sure method is upper-case always
+  slvr_options$method = toupper(slvr_options$method)
+
+  # set default control options
+  if (!("check_hessian" %in% names(control))){
+    control$check_hessian = TRUE
+  }
+  if (!("slvr_log" %in% names(control))){
+    control$slvr_log = FALSE
+  }
+  if (!("running_diagnostics" %in% names(control))){
+    control$running_diagnostics = FALSE
+  }
+
   # Check inputs
   if (!isnull_X){
     if (nrow(as.matrix(X_mu)) != length(y)){
@@ -236,27 +315,36 @@ lmvar <- function( y, X_mu = NULL, X_sigma = NULL, check_hessian = TRUE, slvr_op
   logLik_lm = -n * (log(2*pi*sigma2) + 1) / 2
   logLik_lm = as.numeric(logLik_lm)
 
-  # Set inital estimate of betas if none has been specified
+  # Set inital estimate of betas
   if ("start" %in% names(slvr_options)){
+
     slvr_options$start = slvr_options$start[!aliased_sigma]
-    slvr_default = list(fn = logLHood)
+    slvr_default_start = list()
   }
   else {
-    beta = stats::coef(fit)
+
     sigma = summary(fit)$sigma
     beta_sigma = log(sigma) * solve( Matrix::t(X_sigma) %*% X_sigma) %*% Matrix::colSums(X_sigma)  # Minimize sum over all elements of (X_sigma %*% beta_sigma - log(sigma))^2
-    beta_sigma = as.numeric(beta_sigma)
-    slvr_default = list( fn = logLHood, start = beta_sigma )
+    slvr_default_start = list(start = as.numeric(beta_sigma))
   }
 
+  # set function
+  slvr_default_fn = list(logLik = logLHood)
+
   # Calculate beta for standard deviation sigma
-  solve_result = do.call(maxLik::maxNR, c( slvr_default, slvr_options))
+  solve_result = do.call(maxLik::maxLik, c( slvr_default_fn, slvr_default_start, slvr_options))
 
   # Check on errors and warnings from solver
-  if (!(solve_result$code %in% c( 1, 2, 8))){
+  if (slvr_options$method == "NR")
+    ok_codes = c( 1, 2, 8)
+  else {
+    ok_codes = 0
+    }
+  if (!(solve_result$code %in% ok_codes)){
     warning(solve_result$message)
   }
 
+  # Extract beta_sigma from solve results
   beta_sigma = solve_result$estimate
 
   # Calculate beta for expectation value mu
@@ -266,8 +354,14 @@ lmvar <- function( y, X_mu = NULL, X_sigma = NULL, check_hessian = TRUE, slvr_op
   beta = as.numeric(M %*% Matrix::t(X_mu) %*% (y / (sigma * sigma)))
 
   # Check Hessian
-  if (check_hessian){
-    if(!matrixcalc::is.negative.definite(as.matrix(solve_result$hessian))){
+  if (control$check_hessian){
+    if (slvr_options$method == "NR"){
+      hessian = maxLik::hessian(solve_result)
+    }
+    else {
+      hessian = attr( logLHood( beta_sigma, return_hessian = TRUE), "hessian")
+    }
+    if(!matrixcalc::is.negative.definite(hessian)){
       warning("Log-likelihood appears not to be at a maximum!")
     }
   }
@@ -287,14 +381,16 @@ lmvar <- function( y, X_mu = NULL, X_sigma = NULL, check_hessian = TRUE, slvr_op
                 logLik_lm = logLik_lm,
                 aliased_mu = aliased_mu,
                 aliased_sigma = aliased_sigma,
-                intercept_mu = intercept_mu,
-                intercept_sigma = intercept_sigma,
                 y = y,
                 X_mu = X_mu,
-                X_sigma = X_sigma)
+                X_sigma = X_sigma,
+                intercept_mu = intercept_mu,
+                intercept_sigma = intercept_sigma,
+                slvr_options = slvr_options,
+                control = control)
 
   # Add ouput of solver routine
-  if (slvr_log){
+  if (control$slvr_log){
     rlist$slvr_log = solve_result
   }
 
